#!/usr/bin/env python

"""
Login resources which redirect clients back to an application after a successful
login.

Copyright (C) 2004, 2005, 2006, 2007 Paul Boddie <paul@boddie.org.uk>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""

import WebStack.Generic
from WebStack.Helpers.Auth import LoginAuthenticator

class LoginResource:

    "A resource providing a login screen."

    encoding = "utf-8"

    def __init__(self, authenticator, use_redirect=1, urlencoding=None, encoding=None):

        """
        Initialise the resource with an 'authenticator'.

        If the optional 'use_redirect' flag is set to 0, a confirmation screen
        is given instead of redirecting the user back to the original
        application.

        The optional 'urlencoding' parameter allows a special encoding to be
        used in producing the redirection path.

        The optional 'encoding' parameter allows a special encoding to be used
        in producing the login pages.

        To change the pages employed by this resource, either redefine the
        'login_page' and 'success_page' attributes in instances of this class or
        a subclass, or override the 'show_login' and 'show_success' methods.
        """

        self.authenticator = authenticator
        self.use_redirect = use_redirect
        self.urlencoding = urlencoding
        self.encoding = encoding or self.encoding

    def respond(self, trans):

        "Respond using the transaction 'trans'."

        # Check for a submitted login form.

        fields_body = trans.get_fields_from_body(self.encoding)

        if fields_body.has_key("login"):
            if self.authenticator.authenticate(trans, fields_body.get("username", [None])[0], fields_body.get("password", [None])[0]):
                app = fields_body.get("app", [""])[0]

                self._redirect(trans, app)
                # The above method does not return.

        # Otherwise, show the login form.

        fields_path = trans.get_fields_from_path(self.urlencoding)
        app = fields_path.get("app", [""])[0]

        self.show_login(trans, app)

    def _redirect(self, trans, app):

        """
        Redirect the client using 'trans' and the given 'app' details.
        """

        # Show the success page anyway.

        self.show_success(trans, app)
        if self.use_redirect:
            trans.redirect(app)
        else:
            raise WebStack.Generic.EndOfResponse

    def show_login(self, trans, app):

        """
        Writes a login screen using the transaction 'trans', including details
        of the 'app' which the client was attempting to access.
        """

        trans.set_content_type(WebStack.Generic.ContentType("text/html", self.encoding))
        out = trans.get_response_stream()
        out.write(self.login_page % app)

    def show_success(self, trans, app):

        """
        Writes a success screen using the transaction 'trans', including details
        of the 'app' which the client was attempting to access.
        """

        trans.set_content_type(WebStack.Generic.ContentType("text/html", self.encoding))
        out = trans.get_response_stream()
        out.write(self.success_page % (app, app))

    login_page = """
<html>
  <head>
    <title>Login</title>
  </head>
  <body>
    <h1>Login</h1>
    <form method="POST">
      <p>Username: <input name="username" type="text" size="12"/></p>
      <p>Password: <input name="password" type="password" size="12"/></p>
      <p><input name="login" type="submit" value="Login"/></p>
      <input name="app" type="hidden" value="%s"/>
    </form>
  </body>
</html>
"""

    success_page = """
<html>
  <head>
    <title>Login Example</title>
  </head>
  <body>
    <h1>Login Successful</h1>
    <p>Please proceed to the application: <a href="%s">%s</a></p>
  </body>
</html>
"""

# vim: tabstop=4 expandtab shiftwidth=4
